@@ -3,15 +3,20 @@ import os
 
 from cffi import FFI
 
-libraries=['m', 'z', 'lzma', 'bz2', 'pthread', 'curl', 'crypto']
-library_dirs=[]
+dir_path = os.path.dirname(os.path.realpath(__file__))
 src_dir='src'
+libraries=['m', 'lzma', 'bz2', 'pthread', 'curl', 'crypto']
+library_dirs=[]
+print("WITHDEFLATE:", os.getenv('WITHDEFLATE'))
+if os.getenv('WITHDEFLATE') == "1":
+    print("Using deflate")
+    libraries.append('deflate')
+    library_dirs.append(os.path.join(dir_path, 'libdeflate'))
 
 ffibuilder = FFI()
 ffibuilder.set_source("libmodbampy",
     r"""
     #include "htslib/sam.h"
-    #include "args.h"
     #include "bamiter.h"
     #include "common.h"
     #include "counts.h"
@@ -20,9 +25,6 @@ ffibuilder.set_source("libmodbampy",
     libraries=libraries,
     library_dirs=library_dirs,
     include_dirs=[src_dir, 'htslib'],
-    #sources=[
-    #    os.path.join(src_dir, x)
-    #    for x in ('common.c', 'bamiter.c', 'counts.c', 'args.c')],
     extra_compile_args=['-std=c99', '-msse3', '-O3'],
     extra_objects=[
         'pymod.a',
@@ -40,9 +42,23 @@ cdef = ["""
     // basic bam opening/handling
     typedef struct bam1_core_t {hts_pos_t pos; uint16_t flag; uint32_t l_qseq; ...;} bam1_core_t;
     typedef struct bam1_t {bam1_core_t core; uint8_t *data; ...;} bam1_t;
+    bam1_t *bam_init1();
+    void bam_destroy1(bam1_t *b);
     typedef struct mplp_data {...;} mplp_data;
+
+    // opening bam with idx and hdr info
+    typedef struct { ...; } bam_fset;
+    bam_fset* create_bam_fset(char* fname);
+    void destroy_bam_fset(bam_fset* fset);
+    typedef struct set_fsets {
+        bam_fset **fsets;
+        size_t n;
+    } set_fsets;
+    set_fsets *create_filesets(const char **bams);
+    void destroy_filesets(set_fsets *s);
+
     mplp_data *create_bam_iter_data(
-        const char *bam_file, const char *chr, int start, int end,
+        const bam_fset* fset, const char *chr, int start, int end,
         const char *read_group, const char tag_name[2], const int tag_value);
     void destroy_bam_iter_data(mplp_data *data);
     // iterate a file
@@ -73,7 +89,7 @@ cdef = ["""
 """]
 
 # add in some things from headers, removing directives
-for header in ('src/args.h', 'src/counts.h'):
+for header in ('src/counts.h', ):
     with open(header, 'r') as fh:
         cdef.append("// START: {}".format(header))
         cdef.append(
