@@ -2,6 +2,10 @@
 #define _MODBAMBED_COUNTS_H
 
 #include <stdbool.h>
+#include <limits.h>
+
+
+static const int _INT_MAX = INT_MAX;
 
 // medaka-style feature data
 typedef struct _plp_data {
@@ -13,6 +17,35 @@ typedef struct _plp_data {
 } _plp_data;
 typedef _plp_data *plp_data;
 
+// files open for writing outputs
+typedef struct _output_files {
+    bool multi;
+    bool take_all;
+    bool cpg;
+    bool chh;
+    bool chg;
+    FILE *fcpg;
+    FILE *fchh;
+    FILE *fchg;
+} _output_files;
+typedef _output_files *output_files;
+
+output_files open_bed_files(char* prefix, bool cpg, bool chh, bool chg);
+void close_bed_files(output_files);
+
+// Check sequences for motifs
+// CpG
+bool extern inline is_cpg_fwd(size_t rpos, int rlen, char* ref);
+bool extern inline is_cpg_rev(size_t rpos, int rlen, char* ref);
+// CHN
+bool extern inline _is_chn_fwd(size_t rpos, int rlen, char* ref);
+bool extern inline _is_chn_rev(size_t rpos, int rlen, char* ref);
+// CHH
+bool extern inline is_chh_fwd(size_t rpos, int rlen, char* ref);
+bool extern inline is_chh_rev(size_t rpos, int rlen, char* ref);
+// CHG
+bool extern inline is_chg_fwd(size_t rpos, int rlen, char* ref);
+bool extern inline is_chg_rev(size_t rpos, int rlen, char* ref);
 
 // medaka-style base encoding - augmented with (a) modified base counts
 static const char plp_bases[] = "acgtACGTdDmMfF";  // f for "filtered"
@@ -72,11 +105,11 @@ void print_pileup_data(plp_data pileup);
  *  @param extended whether to include counts of canonical, modified and filtered bases.
  *  @param feature name to use for feature column of BED (e.g. 5mC).
  *  @param canon_base canonical base to match.
- *  @param cpg filter output to only CpG sites.
+ *  @param bed_files output file handles (and filters).
  *  @returns void
  *
  */
-void print_bedmethyl(plp_data pileup, char *ref, int rstart, bool extended, char *feature, char canon_base, bool cpg);
+void print_bedmethyl(plp_data pileup, char *ref, int rstart, bool extended, char *feature, char canon_base, output_files bed_files);
 
 
 /** Generates base counts from a region of a bam.
@@ -91,6 +124,7 @@ void print_bedmethyl(plp_data pileup, char *ref, int rstart, bool extended, char
  *  @param lowthreshold highest probability to call base as canonical.
  *  @param highthreshold lowest probablity to call base as modified.
  *  @param mod_base BAM code for modified base to report. (e.g. h for 5hmC).
+ *  @param max_depth maximum depth of pileup.
  *  @returns a pileup data pointer.
  *
  *  The return value can be freed with destroy_plp_data.
@@ -99,6 +133,6 @@ void print_bedmethyl(plp_data pileup, char *ref, int rstart, bool extended, char
 plp_data calculate_pileup(
     const set_fsets *fsets, const char *chr, int start, int end,
     const char *read_group, const char tag_name[2], const int tag_value,
-    int lowthreshold, int highthreshold, char mod_base);
+    int lowthreshold, int highthreshold, char mod_base, int max_depth);
 
 #endif
